@@ -4,13 +4,10 @@
 -- Authors: Javier Guerra and Andre Carregal
 -- Copyright (c) 2004-2007 Kepler Project
 --
--- $Id: httpd.lua,v 1.38 2007/08/31 21:20:04 carregal Exp $
+-- $Id: httpd.lua,v 1.43 2008/05/23 20:18:58 carregal Exp $
 -----------------------------------------------------------------------------
 
 local url = require "socket.url"
---require "coxpcall"
---pcall  = copcall
---xpcall = coxpcall
 
 module ("xavante.httpd", package.seeall)
 
@@ -69,12 +66,16 @@ end
 
 
 function errorhandler (msg, co, skt)
-	io.stderr:write("xavante error:", msg, co, skt)
+    msg = tostring(msg)
+	io.stderr:write("  Xavante Error: "..msg.."\n", "  "..tostring(co).."\n", "  "..tostring(skt).."\n")
 	skt:send ("HTTP/1.0 200 OK\r\n")
 	skt:send (string.format ("Date: %s\r\n\r\n", os.date ("!%a, %d %b %Y %H:%M:%S GMT")))
 	skt:send (string.format ([[
-<H1>ERROR</H1>
+<html><head><title>Xavante Error!</title></head>
+<body>
+<h1>Xavante Error!</h1>
 <p>%s</p>
+</body></html>
 ]], string.gsub (msg, "\n", "<br/>\n")))
 end
 
@@ -107,7 +108,7 @@ end
 --		req.headers: table of header fields, as name => value
 function read_headers (req)
 	local headers = {}
-	local prevname
+	local prevval, prevname
 	
 	while 1 do
 		local l,err = req.socket:receive ()
@@ -150,14 +151,18 @@ function default_headers (req)
 end
 
 function add_res_header (res, h, v)
-	local prevval = res.headers [h]
-	if (prevval  == nil) then
-		res.headers[h] = v
-	elseif type (prevval) == "table" then
-		table.insert (prevval, v)
-	else
-		res.headers[h] = {prevval, v}
-	end
+    if string.lower(h) == "status" then
+        res.statusline = "HTTP/1.1 "..v
+    else
+        local prevval = res.headers [h]
+        if (prevval  == nil) then
+            res.headers[h] = v
+        elseif type (prevval) == "table" then
+            table.insert (prevval, v)
+        else
+            res.headers[h] = {prevval, v}
+        end
+    end
 end
 
 
@@ -176,10 +181,10 @@ local function send_res_headers (res)
 	if xavante.cookies then
 		xavante.cookies.set_res_cookies (res)
 	end
-		
-	res.statusline = res.statusline or "HTTP/1.1 200 OK\r\n"
+    
+	res.statusline = res.statusline or "HTTP/1.1 200 OK"
 	
-	res.socket:send (res.statusline)
+	res.socket:send (res.statusline.."\r\n")
 	for name, value in pairs (res.headers) do
                 if type(value) == "table" then
                   for _, value in ipairs(value) do
@@ -255,7 +260,7 @@ function send_response (req, res)
 		end
 	else
 		if not res.sent_headers then
-			res.statusline = "HTTP/1.1 204 No Content\r\n"
+			res.statusline = "HTTP/1.1 204 No Content"
 			res.headers["Content-Length"] = 0
 		end
 	end
@@ -264,9 +269,7 @@ function send_response (req, res)
         res:add_header ("Transfer-Encoding", "chunked")
     end
     
-	if res.chunked or 
-		((res.headers ["Content-Length"]) and
-		req.headers ["connection"] == "Keep-Alive")
+	if res.chunked or ((res.headers ["Content-Length"]) and req.headers ["connection"] == "Keep-Alive")
 	then
 		res.headers ["Connection"] = "Keep-Alive"
 		res.keep_alive = true
@@ -315,7 +318,7 @@ function getparams (req)
 end
 
 function err_404 (req, res)
-	res.statusline = "HTTP/1.1 404 Not Found\r\n"
+	res.statusline = "HTTP/1.1 404 Not Found"
 	res.headers ["Content-Type"] = "text/html"
 	res.content = string.format ([[
 <!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">
@@ -329,7 +332,7 @@ The requested URL %s was not found on this server.<P>
 end
 
 function err_405 (req, res)
-	res.statusline = "HTTP/1.1 405 Method Not Allowed\r\n"
+	res.statusline = "HTTP/1.1 405 Method Not Allowed"
 	res.content = string.format ([[
 <!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">
 <HTML><HEAD>
@@ -343,7 +346,7 @@ end
 
 function redirect (res, d)
 	res.headers ["Location"] = d
-	res.statusline = "HTTP/1.1 302 Found\r\n"
+	res.statusline = "HTTP/1.1 302 Found"
 	res.content = "redirect"
 end
 
